%
% function reconstructed_current_frame = MC_CS_PL_Decoder(reference_frame, ...
%   recon_2d, y, Phis, num_rows, num_cols, recon_for_tele)
%
%   This function performs MC-CS-PL method to reconstruct current frame
%   based on reference frame. Multihypothesis initialization of independent
%   reconstruction and residual reconstruction without motion is utilized
%   for the initial reconstruction of current frame to obtain motion
%   vectors. After that, residual reconstruction is performed which enables
%   better reconstruction than direct reconstruction because it is sparser.
%   Predicted frame is found by interpolation between telescopic search MC
%   frame and regular MC frame. 
%   This function returns the MC-CS-PL reconstructed frame.
%
%   See:
%     S. Mun and J. E. Fowler, "Motion-Compensated Compressed-Sensing 
%     Reconstruction for Dynamic MRI," IEEE International Conference on
%     Image Processing, 2013 
%
%   Originally written by SungKwang Mun, Mississippi State University
%

%
% MC-CS-PL: Motion-Compensated Compressed Sensing - Projected Landweber
% Copyright (C) 2013  James E. Fowler
% http://www.ece.mstate.edu/~fowler/BCSPL/
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%

function reconstructed_current_frame = MC_CS_PL_Decoder(reference_frame, ...
    recon_2d, y, Phis, num_rows, num_cols, recon_for_tele)

subrate = Phis{6};

weight = subrate*2 - 0.2;
% weight = 0.5;
weight(weight>0.8) = 0.8;
weight(weight<0.2) = 0.2;

recon_without_motion = ...
    MC_CS_PL_ResidualReconstruction(reference_frame, y, Phis, ...
    num_rows, num_cols);
reconstructed_current_frame = recon_2d*weight + recon_without_motion*(1-weight);

block_size = 32;
window_size = 7;

recon_for_tele = [recon_for_tele {reconstructed_current_frame}];

% telescopic search ME/MC over whole GOP
motion_compensated_frame1=  memc_telescope(recon_for_tele, reconstructed_current_frame, block_size, window_size);
% ME/MC only on the previously reconstructed frame
motion_compensated_frame2=  motion_estimation_compensation(reference_frame, reconstructed_current_frame, block_size, window_size);

motion_compensated_frame = motion_compensated_frame1*weight + motion_compensated_frame2*(1-weight);

% final reconstruction
reconstructed_current_frame = MC_CS_PL_ResidualReconstruction( ...
    motion_compensated_frame, y, Phis, num_rows, num_cols);
