% 
% function reconstructed_center_frame = MC_CS_PL_Decoder_Center( ...
%   y, previous_frame, next_frame, Phis, recon_2d, recon_for_tele, center_frame)
% 
%   This function performs MC-CS-PL method to reconstruct center frame
%   based on two reference frame in bidirecional way. Mulityhypothesis 
%   initialization includes independent reconstruction and two residual
%   reconstruction without motion. Two residual reconstruction of motion 
%   compensated frames composited by adjacent frames are averaged as final
%   reconstruction of center frame. Predicted frame is found by interpolation
%   between telescopic search MC frame and regular MC frame. 
%   This function returns reconstructed frame by MC-CS-PL.
%
%   See:
%     S. Mun and J. E. Fowler, "Motion-Compensated Compressed-Sensing 
%     Reconstruction for Dynamic MRI," IEEE International Conference on
%     Image Processing, 2013 
%
%   Originally written by SungKwang Mun, Mississippi State University
%

%
% MC-CS-PL: Motion-Compensated Compressed Sensing - Projected Landweber
% Copyright (C) 2013  James E. Fowler
% http://www.ece.mstate.edu/~fowler/BCSPL/
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%

function reconstructed_center_frame = MC_CS_PL_Decoder_Center( ...
    y, previous_frame, next_frame, Phis, recon_2d, recon_for_tele)

subrate = Phis{6};
weight = subrate*2 - 0.2;
% weight = 0.5;
weight(weight>0.8) = 0.8;
weight(weight<0.2) = 0.2;

GOP_size = length(recon_for_tele) - 1; 
ind_center = floor(GOP_size/2) + 1;

[num_rows, num_cols] = size(previous_frame);

% Hypothesis Initialization
recon_res_zeroMV1 = ...
    MC_CS_PL_ResidualReconstruction(previous_frame, y, Phis, num_rows, num_cols);
recon_res_zeroMV2 = ...
    MC_CS_PL_ResidualReconstruction(next_frame, y, Phis, num_rows, num_cols);
recon_without_motion = (recon_res_zeroMV1 + recon_res_zeroMV2)/2;

reconstructed_current_frame = recon_2d*weight + recon_without_motion*(1-weight);

% Bidirectional iterative motion compensation
recon_for_tele{ind_center} = reconstructed_current_frame;

block_size = 32;
window_size = 7;
reference_frame = previous_frame;
recon_current_frame_forward = ...
    MC_CS_PL_Reconstruction(reconstructed_current_frame, ...
    reference_frame, y, Phis, block_size, window_size, recon_for_tele, weight); 

reference_frame = next_frame;
recon_current_frame_backward = ...
    MC_CS_PL_Reconstruction(reconstructed_current_frame, ...
    reference_frame, y, Phis, block_size,window_size, recon_for_tele, weight); 
% average result
reconstructed_center_frame = ...
    (recon_current_frame_forward + recon_current_frame_backward)/2;

% Auxiliary function
function reconstructed_frame = ...
    MC_CS_PL_Reconstruction(reconstructed_frame, reference_frame, ...
    y, Phis, block_size,window_size, recon_for_tele, weight)

[num_rows, num_cols] = size(reconstructed_frame);
motion_compensated_image1= memc_telescope(recon_for_tele, reconstructed_frame, block_size, window_size);
motion_compensated_image2 = motion_estimation_compensation(reference_frame, reconstructed_frame, ...
    block_size, window_size);
motion_compensated_image = motion_compensated_image1*weight + motion_compensated_image2*(1-weight);
reconstructed_frame = ...
    MC_CS_PL_ResidualReconstruction(motion_compensated_image,...
    y, Phis, num_rows, num_cols);
