% 
% function [reconstructed_frames reconstructed_frames_indep] = ...
%   MC_CS_PL_Decoder_GOP(y, Phis, GOP_size, num_rows, num_cols)
% 
%   This function performs Forward/Backward reconstruction from the given
%   two key frames to center frame. Center frame is reconstructed by
%   bidirectional reconstruction. y is the measurements of GOP, Phis
%   is the set of measurement operators, GOP_size is the size of the GOP,
%   and num_rows and num_cols are the dimension of the frames.
%   This function returns reconstructed frames by MC-CS-PL and independently 
%   reconstructed frames by CS-PL.
%
%   See:
%     S. Mun and J. E. Fowler, "Motion-Compensated Compressed-Sensing 
%     Reconstruction for Dynamic MRI," IEEE International Conference on
%     Image Processing, 2013 
%
%   Originally written by SungKwang Mun, Mississippi State University
%

%
% MC-CS-PL: Motion-Compensated Compressed Sensing - Projected Landweber
% Copyright (C) 2013  James E. Fowler
% http://www.ece.mstate.edu/~fowler/BCSPL/
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%

function [reconstructed_frames, reconstructed_frames_indep] = ...
    MC_CS_PL_Decoder_GOP(y, Phis, GOP_size, num_rows, num_cols)

Phi_key = Phis{1};
Phi_key_t = Phis{2};
Phi = Phis{3};
Phi_t = Phis{4};

ind_center_frame = GOP_size/2 + 1;

disp('< 2D-CS-PL Initial Reconstruction >');
recon_2d = cell(1,GOP_size+1);
recon_2d{1} = CS_PL_DDWT_Decoder(y{1}, Phi_key, Phi_key_t, num_rows, ...
    num_cols);
for i = 2:GOP_size
  recon_2d{i} = CS_PL_DDWT_Decoder(y{i}, Phi, Phi_t, num_rows, ...
      num_cols);
end
recon_2d{GOP_size+1}= CS_PL_DDWT_Decoder(y{end}, Phi_key, Phi_key_t, ...
    num_rows, num_cols);

reconstructed_frames_indep = recon_2d(1:end-1);

disp('< Forward Processing >');
recon_forward_frames = MC_CS_PL_Decoder_ForwardBackward( ...
    y(1:ind_center_frame-1), Phis, num_rows, num_cols, ...
    recon_2d(2:ind_center_frame-1), recon_2d(1:end));

disp('< Backward Processing >' );
recon_backward_frames = MC_CS_PL_Decoder_ForwardBackward( ...
    y(GOP_size+1:-1:ind_center_frame+1), Phis, num_rows, num_cols, ...
    recon_2d(GOP_size:-1:ind_center_frame+1), recon_2d(end:-1:1));

disp('< Bidirectional Processing for Center frame >');
recon_center_frame = MC_CS_PL_Decoder_Center(y{ind_center_frame}, ...
    recon_forward_frames{end}, recon_backward_frames{end}, Phis, ...
    recon_2d{ind_center_frame}, recon_2d);

reconstructed_frames = ...
    [recon_forward_frames recon_center_frame recon_backward_frames(end:-1:2)];
