% 
% function [y Phi] = MC_CS_PL_Encoder(frames, subrate, key_subrate)
% 
%   This function performs CS projections of each frame in 
%   the group of frames. Based on assumption that the key frame subrate
%   is larger than the non-key frame subrate, two different sampling operators
%   and their adjoint operators are used. The projections and the operator are 
%   returned. Actual subrate can be calculated by the ratio between non-zeros
%   and total elements in y.
%
%   See:
%     S. Mun and J. E. Fowler, "Motion-Compensated Compressed-Sensing 
%     Reconstruction for Dynamic MRI," IEEE International Conference on
%     Image Processing, 2013 
%
%   Originally written by SungKwang Mun, Mississippi State University
%

%
% MC-CS-PL: Motion-Compensated Compressed Sensing - Projected Landweber
% Copyright (C) 2013  James E. Fowler
% http://www.ece.mstate.edu/~fowler/BCSPL/
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%

function [y, Phis] = MC_CS_PL_Encoder(frames, key_subrate, subrate)

[num_rows, num_cols] = size(frames{1});
N = num_rows*num_cols;
num_frames = length(frames);

% key frames
% to determine how many radial lines are needed w.r.t subrate 
L = round((0.6464*key_subrate^2 + 0.8431*key_subrate + 0.014)*num_rows);
mask = random_radial_line_mask(L,num_rows);

Phi_key = @(x) 1/sqrt(N)*fftshift(fft2(x)).*mask;
Phi_key_t=  @(x) sqrt(N)*ifft2(ifftshift(x));

y = cell(1,num_frames);

y{1} = Phi_key(frames{1});

% non-key frames
L = round((0.6464*subrate^2 + 0.8431*subrate + 0.014)*num_rows);
mask = random_radial_line_mask(L,num_rows);
Phi = @(x) 1/sqrt(N)*fftshift(fft2(x)).*mask;
Phi_t=  @(x) sqrt(N)*ifft2(ifftshift(x));

for i = 2 : num_frames
  y{i} = Phi(frames{i});
end

y{end} = Phi_key(frames{end});

Phis = {Phi_key Phi_key_t Phi Phi_t key_subrate subrate};
