% 
% function reconstructed_current_image = MC_CS_PL_ResidualReconstruction( ...
% 	motion_compensated_image, y, Phis, num_rows, num_cols)
% 
%   This function performs SPL reconstruction of residual of y and 
%   projected motion compensated frame using a DDWT sparsity basis. Once 
%   residual is reconstructed it is added back to motion compensated frame
%   that is reconstuced image. Phis gives the projection matrices. 
%   The reconstructed image, of size num_rows x num_cols, is 
%   returned as reconstructed_image. 
%
%   See:
%     S. Mun and J. E. Fowler, "Motion-Compensated Compressed-Sensing 
%     Reconstruction for Dynamic MRI," IEEE International Conference on
%     Image Processing, 2013 
%
%   Originally written by SungKwang Mun, Mississippi State University
%

%
% MC-CS-PL: Motion-Compensated Compressed Sensing - Projected Landweber
% Copyright (C) 2013  James E. Fowler
% http://www.ece.mstate.edu/~fowler/BCSPL/
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%

function reconstructed_current_image = MC_CS_PL_ResidualReconstruction( ...
    motion_compensated_image, y, Phis, num_rows, num_cols)
Phi = Phis{3};
Phi_t = Phis{4};

% projecting the compensated image onto random basis
y_mc = Phi(motion_compensated_image);

% taking the difference to pursue the sparcity.
y_r = y - y_mc;

nonNegative = 0; % because motion compensated image is not necessarily positive
% reconstrucing the residual
x_r = CS_PL_DDWT_Decoder(y_r, Phi, Phi_t, num_rows, num_cols, nonNegative);

reconstructed_current_image = motion_compensated_image + x_r;
