%
% function  motion_compensated_image = motion_estimation_compensation( ...
%   reference_image, current_image, B, W)
%
%   This function performs telescpic search on a reference frame and generate
%   motion compensated frame. 
%
%   See:
%     S. Mun and J. E. Fowler, "Motion-Compensated Compressed-Sensing 
%     Reconstruction for Dynamic MRI," IEEE International Conference on
%     Image Processing, 2013 
%
%   Originally written by SungKwang Mun, Mississippi State University
%

%
% MC-CS-PL: Motion-Compensated Compressed Sensing - Projected Landweber
% Copyright (C) 2013  James E. Fowler
% http://www.ece.mstate.edu/~fowler/BCSPL/
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%
function  motion_compensated_image = motion_estimation_compensation( ...
    reference_image, current_image, B, W)

[num_rows num_cols] = size(current_image);
block_num_rows = floor(num_rows / B);

reference_image = symextend(reference_image,2*W);
current_image = im2col(current_image,[B B],'distinct');

for block_row = 1:B:num_rows
  for block_col = 1:B:num_cols
    
    index = ((block_col-1)/B)*block_num_rows + (block_row-1)/B + 1;
    current_block = current_image(:,index);  
    
    % i,j center of the window
    i = block_row + 2*W;
    j = block_col + 2*W;
    
    % extract all blocks from windows
    window_ref = reference_image(i-W:i+W+B-1,j-W:j+W+B-1);
    H = im2col(window_ref,[B B],'sliding');
    % obtaining mean absolute error for all blocks extracted from the window
    MAEs = mean(abs(bsxfun(@minus, H, current_block)));
    [val ind] = min(MAEs);
    
    motion_compensated_image(block_row:block_row+B-1,block_col:block_col+B-1) = ...
        reshape(H(:,ind),[B B]);
    
  end
end
